@@ -19,6 +19,8 @@
 # Boston, MA 02111-1307, USA.
 #
 
+import copy
+
 from .message import Position
 from .odict import odict
 from .utils import to_underscores
@@ -99,6 +101,13 @@ in contrast to the other create_type() functions."""
             bare_utf8.ctype = None
             return Array(None, bare_utf8, ctype=None, gtype_name=gtype_name,
                          is_const=False)
+
+        # Workaround for Gdk.Rectangle being boxed alias for
+        # cairo.RectangleInt.  G-I does not support boxing of aliases.
+        # See https://bugzilla.gnome.org/show_bug.cgi?id=655423
+        if gtype_name == 'GdkRectangle':
+            gtype_name = 'CairoRectangleInt'
+
         return cls(gtype_name=gtype_name)
 
     def get_giname(self):
@@ -213,6 +222,10 @@ GIR_TYPES = [TYPE_NONE, TYPE_ANY]
 GIR_TYPES.extend(BASIC_GIR_TYPES)
 GIR_TYPES.extend([TYPE_STRING, TYPE_FILENAME, TYPE_VALIST])
 
+# These are the only basic types that are guaranteed to
+# be as big as a pointer (and thus are allowed in GPtrArray)
+POINTER_TYPES = [TYPE_ANY, TYPE_INTPTR, TYPE_UINTPTR]
+
 INTROSPECTABLE_BASIC = list(GIR_TYPES)
 for v in [TYPE_NONE, TYPE_ANY,
           TYPE_LONG_LONG, TYPE_LONG_ULONG,
@@ -314,6 +327,12 @@ PARAM_TRANSFER_NONE = 'none'
 PARAM_TRANSFER_CONTAINER = 'container'
 PARAM_TRANSFER_FULL = 'full'
 
+SIGNAL_FIRST = 'first'
+SIGNAL_LAST = 'last'
+SIGNAL_CLEANUP = 'cleanup'
+SIGNAL_MUST_COLLECT = 'must-collect'
+
+
 class Namespace(object):
     def __init__(self, name, version,
                  identifier_prefixes=None,
@@ -560,6 +579,28 @@ class Function(Callable):
         self.is_constructor = False
         self.shadowed_by = None # C symbol string
         self.shadows = None # C symbol string
+        self.moved_to = None # namespaced function name string
+
+    def clone(self):
+        clone = copy.copy(self)
+        # copy the parameters array so a change to self.parameters does not
+        # influence clone.parameters.
+        clone.parameters = self.parameters[:]
+        return clone
+
+
+class ErrorQuarkFunction(Function):
+
+    def __init__(self, name, retval, parameters, throws, symbol, error_domain):
+        Function.__init__(self, name, retval, parameters, throws, symbol)
+        self.error_domain = error_domain
+
+
+class ErrorQuarkFunction(Function):
+
+    def __init__(self, name, retval, parameters, throws, symbol, error_domain):
+        Function.__init__(self, name, retval, parameters, throws, symbol)
+        self.error_domain = error_domain
 
 
 class VFunction(Callable):
@@ -697,8 +738,13 @@ class Enum(Node, Registered):
         self.c_symbol_prefix = c_symbol_prefix
         self.ctype = ctype
         self.members = members
-        # Associated error quark
-        self.error_quark = None
+        # Associated error domain name
+        self.error_domain = None
+        self.static_methods = []
+
+    def _walk(self, callback, chain):
+        for meth in self.static_methods:
+            meth.walk(callback, chain)
 
 
 class Bitfield(Node, Registered):
@@ -713,6 +759,11 @@ class Bitfield(Node, Registered):
         self.ctype = ctype
         self.c_symbol_prefix = c_symbol_prefix
         self.members = members
+        self.static_methods = []
+
+    def _walk(self, callback, chain):
+        for meth in self.static_methods:
+            meth.walk(callback, chain)
 
 
 class Member(Annotated):
@@ -844,8 +895,15 @@ class Boxed(Node, Registered):
 
 class Signal(Callable):
 
-    def __init__(self, name, retval, parameters):
+    def __init__(self, name, retval, parameters, when=None,
+                 no_recurse=False, detailed=False, action=False,
+                 no_hooks=False):
         Callable.__init__(self, name, retval, parameters, False)
+        self.when = when
+        self.no_recurse = no_recurse
+        self.detailed = detailed
+        self.action = action
+        self.no_hooks = no_hooks
 
 
 class Class(Node, Registered):
@@ -935,10 +993,11 @@ class Interface(Node, Registered):
 
 class Constant(Node):
 
-    def __init__(self, name, value_type, value):
+    def __init__(self, name, value_type, value, ctype):
         Node.__init__(self, name)
         self.value_type = value_type
         self.value = value
+        self.ctype = ctype
 
 
 class Property(Node):
