@@ -56,6 +56,14 @@ def get_preprocessor_option_group(parser):
     group.add_option("-p", dest="", help="Ignored")
     return group
 
+def get_windows_option_group(parser):
+    group = optparse.OptionGroup(parser, "Machine Dependent Options")
+    group.add_option("-m", help="some machine dependent option",
+                     action="append", dest='m_option',
+                     default=[])
+
+    return group
+
 def _get_option_parser():
     parser = optparse.OptionParser('%prog [options] sources')
     parser.add_option('', "--quiet",
@@ -151,6 +159,10 @@ match the namespace prefix.""")
     group = get_preprocessor_option_group(parser)
     parser.add_option_group(group)
 
+    if os.environ.get('MSYSTEM') == 'MINGW32':
+        group = get_windows_option_group(parser)
+        parser.add_option_group(group)
+
     # Private options
     parser.add_option('', "--generate-typelib-tests",
                       action="store", dest="test_codegen", default=None,
@@ -302,7 +314,8 @@ def create_binary(transformer, options, args):
         binary = IntrospectionBinary(args)
     else:
         binary = compile_introspection_binary(options,
-                                              gdump_parser.get_get_type_functions())
+                                              gdump_parser.get_get_type_functions(),
+                                              gdump_parser.get_error_quark_functions())
 
     shlibs = resolve_shlibs(options, binary, options.libraries)
     gdump_parser.set_introspection_binary(binary)
@@ -395,14 +408,15 @@ def scanner_main(args):
 
     ss = create_source_scanner(options, args)
 
+    ap = AnnotationParser()
+    blocks = ap.parse(ss.get_comments())
+
     # Transform the C symbols into AST nodes
+    transformer.set_annotations(blocks)
     transformer.parse(ss.get_symbols())
 
     shlibs = create_binary(transformer, options, args)
 
-    ap = AnnotationParser()
-    blocks = ap.parse(ss.get_comments())
-
     main = MainTransformer(transformer, blocks)
     main.transform()
 
